// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Marc Sturm $
// --------------------------------------------------------------------------

#include <OpenMS/FORMAT/FeaturePairsXMLFile.h>
#include <OpenMS/FORMAT/FeatureXMLFile.h>
#include <OpenMS/ANALYSIS/MAPMATCHING/PairMatcher.h>
#include <OpenMS/KERNEL/ComparatorUtils.h>
#include <OpenMS/KERNEL/FeatureMap.h>

#include <OpenMS/APPLICATIONS/TOPPBase.h>

#include <iostream>
#include <fstream>
#include <string>

using namespace OpenMS;
using namespace std;

//-------------------------------------------------------------
//Doxygen docu
//-------------------------------------------------------------

/**
	@page LabeledMatcher LabeledMatcher
 
	@brief Executes the pair matching algorithm for labeled peptides.
 
	This module identifies pairs of isotope-labeled features in a LC/MS features map.
	By feature, we understand a peptide in a MS sample that reveals a characteristic isotope distribution.

*/

// We do not want this class to show up in the docu:
/// @cond TOPPCLASSES


class TOPPLabeledMatcher
      : public TOPPBase
{
	public:
		
	  TOPPLabeledMatcher()
	    : TOPPBase("LabeledMatcher","find pairs of labeled features in LC/MS data")
	  {
	  }

	protected:
	
	  void registerOptionsAndFlags_()
	  {
			registerInputFile_("in","<file>","","input feature file ");
			setValidFormats_("in",StringList::create("FeatureXML"));
			registerOutputFile_("out","<file>","","output file ");
			setValidFormats_("out",StringList::create("FeaturePairsXML"));
			registerOutputFile_("best","<file>","","output file containing the best pairs only.\n", false);
			setValidFormats_("best",StringList::create("FeaturePairsXML"));
	  	addEmptyLine_();
	  	addText_("Note: RT and m/z shifts and ranges can only be given in the 'algorithm' part of INI file.");
			registerSubsection_("algorithm","Position shift parameters");
	  }
		
		Param getSubsectionDefaults_(const String& /*section*/) const
		{
			FeatureMap<> fm;
			return PairMatcher(fm).getDefaults();
		}
		
	  ExitCodes main_(int , const char**)
	  {
	
	    //-------------------------------------------------------------
	    // parameter handling
	    //-------------------------------------------------------------
	
	    // input file to be read
	    String inputfile = "";
	
	    // determine name of input file
	    inputfile = getStringOption_("in");
	    String outputfile = getStringOption_("out");
	
	    // determine name ouf visualization output file
	    String best_outputfile = getStringOption_("best");
	
	    //-------------------------------------------------------------
	    // reading input
	    //-------------------------------------------------------------
	
	
	    FeatureMap<> features;
	    FeatureXMLFile().load(inputfile,features);
	
	    sort(features.begin(),features.end(), LexicographicComparator<Feature::LessRT,Feature::LessMZ>());
	
	    PairMatcher pm(features);
	
	    Param pm_param = getParam_().copy("algorithm:",true);
	    writeDebug_("Parameters passed to PairMatcher", pm_param, 3);
	    pm.setParameters(pm_param);
	
	    writeDebug_(" Running LabeledMatcher...",1);
	
      const std::vector < ElementPair < Feature > >* pairs = &pm.run();
	
	    //-------------------------------------------------------------
	    // writing files
	    //-------------------------------------------------------------
	
	    writeDebug_(String(" Writing results to ") + outputfile, 1 );
	    FeaturePairsXMLFile().store(outputfile,*pairs);
			
			writeDebug_(String(" Writing results to ") + best_outputfile, 1 );
			if (best_outputfile!="")
	    {
	    	FeaturePairsXMLFile().store(best_outputfile,pm.getBestPairs());
	    }
			
	    return EXECUTION_OK;
	  }
};


int main( int argc, const char** argv )
{
  TOPPLabeledMatcher tool;
  return tool.main(argc,argv);
}

/// @endcond
