// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework 
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Marc Sturm $
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>

///////////////////////////

#include <OpenMS/DATASTRUCTURES/DataValue.h>
#include <sstream>

///////////////////////////

START_TEST(DataValue, "$Id: DataValue_test.C 2734 2008-02-12 12:01:25Z marc_sturm $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

using namespace OpenMS;

// default ctor
DataValue* dv_ptr = 0;
CHECK((DataValue()))
	dv_ptr = new DataValue;
	TEST_NOT_EQUAL(dv_ptr, 0)
RESULT

// destructor
CHECK((~DataValue()))
	delete dv_ptr;
RESULT

// ctor for all supported types a DataValue object can hold

CHECK((DataValue(DoubleReal)))
	DoubleReal x = -3.0;
	DataValue d(x);
	TEST_REAL_EQUAL((DoubleReal)d, -3.0)
RESULT

CHECK((DataValue(Real)))
	Real x = 3.0;
	DataValue d(x);
	TEST_REAL_EQUAL((Real)d, 3.0)
RESULT

CHECK((DataValue(Int)))
	Int n = -3000;
	DataValue d(n);
	TEST_EQUAL((Int)d, -3000)
RESULT

CHECK((DataValue(UInt)))
	UInt n = 3000u;
	DataValue d(n);
	TEST_EQUAL((UInt)d, 3000u)
RESULT

CHECK((DataValue(const char*)))
	const char* s = "test char";
	DataValue d(s);
	TEST_EQUAL((std::string)d, "test char")
RESULT

CHECK((DataValue(const String&)))
	String s = "test string";
	DataValue d(s);
	TEST_EQUAL((String)d, "test string")
RESULT

// copy ctor

CHECK((DataValue(const DataValue&)))
	DataValue p1((DoubleReal) 1.23);
	DataValue p3((Real) 1.23);
	DataValue p4((Int) -3);
	DataValue p5((UInt) 123);
	DataValue p6("test char");
	DataValue p7(std::string("test string"));
	DataValue p8;
	DataValue copy_of_p1(p1);
	DataValue copy_of_p3(p3);
	DataValue copy_of_p4(p4);
	DataValue copy_of_p5(p5);
	DataValue copy_of_p6(p6);
	DataValue copy_of_p7(p7);
	DataValue copy_of_p8(p8);
	TEST_REAL_EQUAL( (DoubleReal) copy_of_p1, 1.23)
	TEST_REAL_EQUAL( (Real) copy_of_p3, 1.23)
	TEST_EQUAL( (Int) copy_of_p4, -3)
	TEST_REAL_EQUAL( (UInt) copy_of_p5, 123)
	TEST_EQUAL( (std::string) copy_of_p6, "test char")
	TEST_EQUAL( (std::string) copy_of_p7, "test string")
	TEST_EQUAL( (copy_of_p8.isEmpty()),true)
RESULT

// assignment operator

CHECK((DataValue& operator = (const DataValue&)))
	DataValue p1((DoubleReal) 1.23);
	DataValue p3((Real) 1.23);
	DataValue p4((Int) -3);
	DataValue p5((UInt) 123);
	DataValue p6("test char");
	DataValue p7(std::string("test string"));
	DataValue p8;
	DataValue copy_of_p;
	copy_of_p = p1;
	TEST_REAL_EQUAL( (DoubleReal) copy_of_p, 1.23)
	copy_of_p = p3;
	TEST_REAL_EQUAL( (Real) copy_of_p, 1.23)
	copy_of_p = p4;
	TEST_EQUAL( (Int) copy_of_p, -3)
	copy_of_p = p5;
	TEST_REAL_EQUAL( (UInt) copy_of_p, 123)
	copy_of_p = p6;
	TEST_EQUAL( (std::string) copy_of_p, "test char")
	copy_of_p = p7;
	TEST_EQUAL( (std::string) copy_of_p, "test string")
	copy_of_p = p8;
	TEST_EQUAL( (copy_of_p.isEmpty()),true)
RESULT

// Is DataValue object empty?

CHECK((bool isEmpty() const))
	DataValue p1;
	bool res1 =  p1.isEmpty();
	TEST_NOT_EQUAL( res1, 0)
	DataValue p2((Real)1.2);
	bool res2 =  p2.isEmpty();
	TEST_EQUAL( res2, 0)
	TEST_REAL_EQUAL( (Real) p2, 1.2)
	DataValue p4("2");
	bool res4 =  p4.isEmpty();
	TEST_EQUAL( res4, 0)
	TEST_EQUAL( (std::string) p4, "2")
RESULT

// conversion operators

CHECK((operator std::string() const throw(Exception::ConversionError)))
	DataValue d((std::string) "test string");
	std::string k = d;
	TEST_EQUAL(k,"test string")
RESULT

CHECK((operator DoubleReal() const throw(Exception::ConversionError)))
	DataValue d((DoubleReal) 5.5);
	DoubleReal k = d;
	TEST_REAL_EQUAL(k,5.5)

  TEST_EXCEPTION(Exception::ConversionError, (UInt)DataValue(-55))
RESULT

CHECK((operator Real() const throw(Exception::ConversionError)))
	DataValue d((Real) 5.45);
	Real k = d;
	TEST_REAL_EQUAL(k,5.45)

  TEST_EXCEPTION(Exception::ConversionError, (UInt)DataValue(-55))
RESULT

CHECK((operator Int() const throw(Exception::ConversionError)))
	DataValue d((Int) 55);
	int k = d;
	TEST_EQUAL(k,55)

  TEST_EXCEPTION(Exception::ConversionError, (UInt)DataValue(-55.4))
RESULT

CHECK((operator UInt() const throw(Exception::ConversionError)))
	DataValue d((Int) 55);
	UInt k = d;
	TEST_EQUAL(k,55)
	
  TEST_EXCEPTION(Exception::ConversionError, (UInt)DataValue(-55))
  TEST_EXCEPTION(Exception::ConversionError, (UInt)DataValue(-55.4))
RESULT

CHECK(([EXTRA] friend bool operator==(const DataValue&, const DataValue&)))
  DataValue a(5.0);
  DataValue b(5.0);
  TEST_EQUAL(a==b,true);
  a = DataValue((DoubleReal)15.13);
  b = DataValue((DoubleReal)15.13);
  TEST_EQUAL(a==b,true);
  a = DataValue((Real)15.13);
  b = DataValue((Real)(17-1.87));
  TEST_EQUAL(a==b,true);
  a = DataValue((Int)5);
  b = DataValue((Int)5);
  TEST_EQUAL(a==b,true);
  a = DataValue((UInt)5000);
  b = DataValue((UInt)5000);
  TEST_EQUAL(a==b,true);
  a = DataValue("hello");
  b = DataValue(std::string("hello"));
  TEST_EQUAL(a==b,true);
  a = DataValue((Real)15.13);
  b = DataValue((Real)(15.13001));
  TEST_EQUAL(a==b,false);
RESULT

CHECK(([EXTRA] friend bool operator!=(const DataValue&, const DataValue&)))
  DataValue a(5.0);
  DataValue b(5.1);
  TEST_EQUAL(a!=b,true);
  a = DataValue((DoubleReal)15.13001);
  b = DataValue((DoubleReal)15.13);
  TEST_EQUAL(a!=b,true);

RESULT

CHECK((const char* toChar() const throw(Exception::ConversionError)))
	DataValue a;
  TEST_EQUAL(a.toChar() == NULL, true)  
  a = DataValue("hello");
  TEST_EQUAL(a.toChar(),std::string("hello"))
	a = DataValue(5);
  TEST_EXCEPTION(Exception::ConversionError, a.toChar() )
RESULT

CHECK((String toString() const))
	DataValue a;
  TEST_EQUAL(a.toString(), "")  
  a = DataValue("hello");
  TEST_EQUAL(a.toString(),"hello")
	a = DataValue(5);
  TEST_EQUAL(a.toString(), "5")
  a = DataValue(47.11);
  TEST_EQUAL(a.toString(), "47.11")
  a = DataValue(-23456.78);
  TEST_EQUAL(a.toString(), "-23456.78")
RESULT

CHECK((QString toQString() const))
	DataValue a;
  TEST_EQUAL(a.toQString().toStdString(), "")  
  a = DataValue("hello");
  TEST_EQUAL(a.toQString().toStdString(),"hello")
	a = DataValue(5);
  TEST_EQUAL(a.toQString().toStdString(), "5")
  a = DataValue(47.11);
  TEST_EQUAL(a.toQString().toStdString(), "47.110000")
  a = DataValue(-23456.78);
  TEST_EQUAL(a.toQString().toStdString(), "-23456.780000")
RESULT

CHECK(([EXTRA] friend std::ostream& operator<<(std::ostream&, const DataValue&)))
	DataValue a((Int)5), b((UInt)100), c((DoubleReal)1.111), d((Real)1.1), e("hello "), f(std::string("world")), g;
	std::ostringstream os;
  os << a << b << c << d << e << f << g;
  TEST_EQUAL(os.str(),"51001.1111.1hello world")
RESULT

CHECK((DataType valueType() const))
	DataValue a;
	TEST_EQUAL(a.valueType(), DataValue::EMPTY_VALUE);

	DataValue a1(1.45);
	TEST_EQUAL(a1.valueType(), DataValue::DOUBLE_VALUE);

	DataValue a2(1.34f);
	TEST_EQUAL(a2.valueType(), DataValue::DOUBLE_VALUE);

	DataValue a3(123);
	TEST_EQUAL(a3.valueType(), DataValue::INT_VALUE);

	DataValue a4("bla");
	TEST_EQUAL(a4.valueType(), DataValue::STRING_VALUE);

	DataValue a5(UInt(2));
	TEST_EQUAL(a5.valueType(), DataValue::INT_VALUE);
RESULT

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
END_TEST
