// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Marc Sturm $
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>

///////////////////////////
#include <OpenMS/FORMAT/PeakFileOptions.h>
///////////////////////////

using namespace OpenMS;
using namespace std;

template<class T>
ostream& operator<<(ostream& os, const vector<T>& vec)
{
	if (vec.empty())
	{
		os << "()";
		return os;
	}
	
	os << "(";
	typename vector<T>::const_iterator i = vec.begin();
	while (true)
	{
		os << *i++;
		if (i == vec.end())
			break;
		os << ",";
	}
	os << ")";
	return os;
}

DRange<1> makeRange(float a, float b)
{
	DPosition<1> pa(a), pb(b);
	return DRange<1>(pa, pb);
}

START_TEST(PeakFileOptions, "$Id: PeakFileOptions_test.C 2734 2008-02-12 12:01:25Z marc_sturm $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

PeakFileOptions* ptr = 0;
CHECK(PeakFileOptions())
	ptr = new PeakFileOptions();
	TEST_NOT_EQUAL(ptr, 0)
RESULT

CHECK(~PeakFileOptions())
	delete ptr;
RESULT

CHECK(void setMetadataOnly(bool only))
	PeakFileOptions tmp;
	tmp.setMetadataOnly(true);
	TEST_EQUAL(tmp.getMetadataOnly(), true);
RESULT

CHECK(bool getMetadataOnly() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.getMetadataOnly(), false);
RESULT

CHECK(void setWriteSupplementalData(bool write))
	PeakFileOptions tmp;
	tmp.setWriteSupplementalData(false);
	TEST_EQUAL(tmp.getWriteSupplementalData(), false);
RESULT

CHECK(bool getWriteSupplementalData() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.getWriteSupplementalData(), true);
RESULT

CHECK(void setRTRange(const DRange<1>& range))
	PeakFileOptions tmp;
	tmp.setRTRange(makeRange(2, 4));
	TEST_EQUAL(tmp.hasRTRange(), true);
	TEST_EQUAL(tmp.getRTRange(), makeRange(2, 4));
RESULT

CHECK(bool hasRTRange() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.hasRTRange(), false);
RESULT

CHECK(const DRange<1>& getRTRange() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.getRTRange(), DRange<1>());
RESULT

CHECK(void setMZRange(const DRange<1>& range))
	PeakFileOptions tmp;
	tmp.setMZRange(makeRange(3, 5));
	TEST_EQUAL(tmp.hasMZRange(), true);
	TEST_EQUAL(tmp.getMZRange(), makeRange(3, 5));
RESULT

CHECK(bool hasMZRange() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.hasMZRange(), false);
RESULT

CHECK(const DRange<1>& getMZRange() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.getMZRange(), DRange<1>());
RESULT

CHECK(void setIntensityRange(const DRange<1>& range))
	PeakFileOptions tmp;
	tmp.setIntensityRange(makeRange(3, 5));
	TEST_EQUAL(tmp.hasIntensityRange(), true);
	TEST_EQUAL(tmp.getIntensityRange(), makeRange(3, 5));
RESULT

CHECK(bool hasIntensityRange() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.hasIntensityRange(), false);
RESULT

CHECK(const DRange<1>& getIntensityRange() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.getIntensityRange(), DRange<1>());
RESULT

CHECK(void setMSLevels(const MSLevels& levels))
	PeakFileOptions tmp;
	PeakFileOptions::MSLevels levels;
	levels.push_back(1);
	levels.push_back(3);
	levels.push_back(5);
	tmp.setMSLevels(levels);
	TEST_EQUAL(tmp.hasMSLevels(), true);
	TEST_EQUAL(tmp.getMSLevels(), levels);
RESULT

CHECK(void addMSLevel(int level))
	PeakFileOptions tmp;
	tmp.addMSLevel(1);
	tmp.addMSLevel(3);
	tmp.addMSLevel(5);
	TEST_EQUAL(tmp.hasMSLevels(), true);
	TEST_EQUAL(tmp.getMSLevels().size(), 3);

	PeakFileOptions::MSLevels levels;
	levels.push_back(1);
	levels.push_back(3);
	levels.push_back(5);

	TEST_EQUAL(tmp.getMSLevels(), levels);
RESULT

CHECK(void clearMSLevels())
	PeakFileOptions tmp;
	PeakFileOptions::MSLevels levels;
	levels.push_back(1);
	levels.push_back(3);
	levels.push_back(5);
	tmp.setMSLevels(levels);
	TEST_EQUAL(tmp.getMSLevels(), levels);
	
	// now clear the ms levels
	tmp.clearMSLevels();
	TEST_EQUAL(tmp.hasMSLevels(), false);
	TEST_EQUAL(tmp.getMSLevels(), PeakFileOptions::MSLevels());
RESULT

CHECK(bool hasMSLevels() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.hasMSLevels(), false);
RESULT

CHECK(bool containsMSLevel(int level) const)
	PeakFileOptions tmp;
	PeakFileOptions::MSLevels levels;
	levels.push_back(1);
	levels.push_back(3);
	levels.push_back(5);
	tmp.setMSLevels(levels);
	TEST_EQUAL(tmp.containsMSLevel(3), true);
	TEST_EQUAL(tmp.containsMSLevel(2), false);
RESULT

CHECK(const MSLevels& getMSLevels() const)
	PeakFileOptions tmp;
	TEST_EQUAL(tmp.getMSLevels(), PeakFileOptions::MSLevels());
RESULT

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
END_TEST
